@@ -1,9 +1,9 @@
 # --
-# Copyright (C) 2001-2018 OTRS AG, https://otrs.com/
+# Copyright (C) 2001-2018 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::System::SupportDataCollector::Plugin::OTRS::DatabaseRecords;
@@ -11,14 +11,13 @@ package Kernel::System::SupportDataCollector::Plugin::OTRS::DatabaseRecords;
 use strict;
 use warnings;
 
-use base qw(Kernel::System::SupportDataCollector::PluginBase);
+use parent qw(Kernel::System::SupportDataCollector::PluginBase);
 
 use Kernel::Language qw(Translatable);
 
 our @ObjectDependencies = (
     'Kernel::Config',
     'Kernel::System::DB',
-    'Kernel::System::Ticket',
 );
 
 sub GetDisplayPath {
@@ -46,7 +45,7 @@ sub Run {
         },
         {
             SQL =>
-                "SELECT count(*) from article_attachment WHERE content_type NOT LIKE 'text/html%'",
+                "SELECT count(*) from article_data_mime_attachment WHERE content_type NOT LIKE 'text/html%'",
             Identifier => 'AttachmentCountDBNonHTML',
             Label      => Translatable("Attachments (DB, Without HTML)"),
         },
@@ -131,7 +130,7 @@ sub Run {
     my $DBObject = $Kernel::OM->Get('Kernel::System::DB');
 
     my %Counts;
-    CHECK:
+    CHECK_RECORDS:
     for my $Check (@Checks) {
         $DBObject->Prepare( SQL => $Check->{SQL} );
         while ( my @Row = $DBObject->FetchrowArray() ) {
@@ -156,39 +155,38 @@ sub Run {
     }
 
     $DBObject->Prepare(
-        SQL => "SELECT max(create_time_unix), min(create_time_unix) FROM ticket WHERE id > 1 ",
+        SQL => "SELECT max(create_time), min(create_time) FROM ticket WHERE id > 1 ",
     );
     my $TicketWindowTime = 1;
     while ( my @Row = $DBObject->FetchrowArray() ) {
         if ( $Row[0] && $Row[1] ) {
-            $TicketWindowTime = ( $Row[0] - $Row[1] ) || 1;
+            my $OldestCreateTimeObject = $Kernel::OM->Create(
+                'Kernel::System::DateTime',
+                ObjectParams => {
+                    String => $Row[0],
+                },
+            );
+            my $NewestCreateTimeObject = $Kernel::OM->Create(
+                'Kernel::System::DateTime',
+                ObjectParams => {
+                    String => $Row[1],
+                },
+            );
+            my $Delta = $NewestCreateTimeObject->Delta( DateTimeObject => $OldestCreateTimeObject );
+            $TicketWindowTime = ( $Delta->{Years} * 12 ) + $Delta->{Months};
         }
-
     }
-    $TicketWindowTime = $TicketWindowTime / ( 60 * 60 * 24 * 30.4 );    # month in seconds
     $TicketWindowTime = 1 if $TicketWindowTime < 1;
 
     $Self->AddResultInformation(
         Identifier => 'TicketWindowTime',
         Label      => Translatable('Months Between First And Last Ticket'),
-        Value      => sprintf( "%.02f", $TicketWindowTime ),
+        Value      => $TicketWindowTime,
     );
     $Self->AddResultInformation(
         Identifier => 'TicketsPerMonth',
         Label      => Translatable('Tickets Per Month (avg)'),
-        Value      => sprintf( "%.02f", $Counts{TicketCount} / $TicketWindowTime ),
-    );
-
-    my $OpenTickets = $Kernel::OM->Get('Kernel::System::Ticket')->TicketSearch(
-        Result    => 'COUNT',
-        StateType => 'Open',
-        UserID    => 1,
-    ) || 0;
-
-    $Self->AddResultInformation(
-        Identifier => 'TicketOpenCount',
-        Label      => Translatable('Open Tickets'),
-        Value      => $OpenTickets,
+        Value      => sprintf( "%d", $Counts{TicketCount} / $TicketWindowTime ),
     );
 
     return $Self->GetResults();
